
#line 3 "lex.calc.c"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

#define yy_create_buffer calc_create_buffer
#define yy_delete_buffer calc_delete_buffer
#define yy_flex_debug calc_flex_debug
#define yy_init_buffer calc_init_buffer
#define yy_flush_buffer calc_flush_buffer
#define yy_load_buffer_state calc_load_buffer_state
#define yy_switch_to_buffer calc_switch_to_buffer
#define yyin calcin
#define yyleng calcleng
#define yylex calclex
#define yylineno calclineno
#define yyout calcout
#define yyrestart calcrestart
#define yytext calctext
#define yywrap calcwrap
#define yyalloc calcalloc
#define yyrealloc calcrealloc
#define yyfree calcfree

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5
#define YY_FLEX_SUBMINOR_VERSION 33
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

/* end standard C headers. */

/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types. 
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t; 
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;
#endif /* ! C99 */

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#endif /* ! FLEXINT_H */

#ifdef __cplusplus

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else	/* ! __cplusplus */

#if __STDC__

#define YY_USE_CONST

#endif	/* __STDC__ */
#endif	/* ! __cplusplus */

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN (yy_start) = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START (((yy_start) - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE calcrestart(calcin  )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#define YY_BUF_SIZE 16384
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

extern int calcleng;

extern FILE *calcin, *calcout;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

    #define YY_LESS_LINENO(n)
    
/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up calctext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		*yy_cp = (yy_hold_char); \
		YY_RESTORE_YY_MORE_OFFSET \
		(yy_c_buf_p) = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up calctext again */ \
		} \
	while ( 0 )

#define unput(c) yyunput( c, (calctext_ptr)  )

/* The following is because we cannot portably get our hands on size_t
 * (without autoconf's help, which isn't available because we want
 * flex-generated scanners to compile on their own).
 */

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef unsigned int yy_size_t;
#endif

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state
	{
	FILE *yy_input_file;

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	yy_size_t yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */
    
	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via calcrestart()), so that the user can continue scanning by
	 * just pointing calcin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2

	};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* Stack of input buffers. */
static size_t yy_buffer_stack_top = 0; /**< index of top of stack. */
static size_t yy_buffer_stack_max = 0; /**< capacity of stack. */
static YY_BUFFER_STATE * yy_buffer_stack = 0; /**< Stack as an array. */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( (yy_buffer_stack) \
                          ? (yy_buffer_stack)[(yy_buffer_stack_top)] \
                          : NULL)

/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE (yy_buffer_stack)[(yy_buffer_stack_top)]

/* yy_hold_char holds the character lost when calctext is formed. */
static char yy_hold_char;
static int yy_n_chars;		/* number of characters read into yy_ch_buf */
int calcleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = (char *) 0;
static int yy_init = 0;		/* whether we need to initialize */
static int yy_start = 0;	/* start state number */

/* Flag which is used to allow calcwrap()'s to do buffer switches
 * instead of setting up a fresh calcin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;

void calcrestart (FILE *input_file  );
void calc_switch_to_buffer (YY_BUFFER_STATE new_buffer  );
YY_BUFFER_STATE calc_create_buffer (FILE *file,int size  );
void calc_delete_buffer (YY_BUFFER_STATE b  );
void calc_flush_buffer (YY_BUFFER_STATE b  );
void calcpush_buffer_state (YY_BUFFER_STATE new_buffer  );
void calcpop_buffer_state (void );

static void calcensure_buffer_stack (void );
static void calc_load_buffer_state (void );
static void calc_init_buffer (YY_BUFFER_STATE b,FILE *file  );

#define YY_FLUSH_BUFFER calc_flush_buffer(YY_CURRENT_BUFFER )

YY_BUFFER_STATE calc_scan_buffer (char *base,yy_size_t size  );
YY_BUFFER_STATE calc_scan_string (yyconst char *yy_str  );
YY_BUFFER_STATE calc_scan_bytes (yyconst char *bytes,int len  );

void *calcalloc (yy_size_t  );
void *calcrealloc (void *,yy_size_t  );
void calcfree (void *  );

#define yy_new_buffer calc_create_buffer

#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){ \
        calcensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            calc_create_buffer(calcin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
	}

#define yy_set_bol(at_bol) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){\
        calcensure_buffer_stack (); \
		YY_CURRENT_BUFFER_LVALUE =    \
            calc_create_buffer(calcin,YY_BUF_SIZE ); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
	}

#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* Begin user sect3 */

typedef unsigned char YY_CHAR;

FILE *calcin = (FILE *) 0, *calcout = (FILE *) 0;

typedef int yy_state_type;

extern int calclineno;

int calclineno = 1;

extern char calctext[];

static yy_state_type yy_get_previous_state (void );
static yy_state_type yy_try_NUL_trans (yy_state_type current_state  );
static int yy_get_next_buffer (void );
static void yy_fatal_error (yyconst char msg[]  );

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up calctext.
 */
#define YY_DO_BEFORE_ACTION \
	(calctext_ptr) = yy_bp; \
	calcleng = (size_t) (yy_cp - yy_bp); \
	(yy_hold_char) = *yy_cp; \
	*yy_cp = '\0'; \
	if ( calcleng >= YYLMAX ) \
		YY_FATAL_ERROR( "token too large, exceeds YYLMAX" ); \
	yy_flex_strncpy( calctext, (calctext_ptr), calcleng + 1 ); \
	(yy_c_buf_p) = yy_cp;

#define YY_NUM_RULES 28
#define YY_END_OF_BUFFER 29
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct yy_trans_info
	{
	flex_int32_t yy_verify;
	flex_int32_t yy_nxt;
	};
static yyconst flex_int16_t yy_accept[228] =
    {   0,
       26,   26,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,   29,   27,   26,   25,   13,   27,    6,   27,
       27,   27,   27,   27,   27,   27,   27,   27,   27,   27,
       27,   26,   12,   28,    9,    8,   19,   18,   19,   15,
       14,   23,   24,   22,   26,    0,    0,    0,    0,    0,
       16,    6,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,   26,   12,    0,
        0,    0,   17,   22,    0,    0,    6,    0,    0,    0,
        0,    0,    5,    0,    0,    0,    7,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,

        0,    0,    0,    0,    0,   22,    4,    0,    0,    0,
        0,    0,    2,    0,    0,    3,   20,    0,    1,    0,
        0,   20,   20,    0,    0,    0,    0,    0,    0,   20,
       20,   20,   20,    0,    0,    0,   21,   11,    0,   20,
       20,   20,   20,   20,    0,    0,   20,   20,   20,   20,
        0,    0,   20,    0,    0,   11,   11,    0,    0,    0,
       20,    0,   20,   20,   20,   20,   20,   20,   20,    0,
        0,   20,   20,   20,   20,    0,    0,    0,    0,   10,
        0,   10,    0,   20,   20,   20,   20,    0,   20,   20,
       20,   20,   20,   20,   20,    0,    0,   20,   20,   20,

        0,    0,   20,   20,   20,   20,    0,   20,   20,   20,
       20,   20,   20,   20,    0,   20,   20,   20,   20,    0,
       20,   20,    0,   20,   20,   20,    0
    } ;

static yyconst flex_int32_t yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    2,    1,    4,    5,    1,    1,    1,    1,    1,
        1,    6,    7,    8,    9,   10,   11,   12,   13,   13,
       13,   13,   13,   13,   13,   13,   13,   14,   15,   16,
        1,   17,    1,   18,   22,   23,   24,   25,   26,   27,
       28,   29,   30,   31,   31,   32,   31,   33,   34,   35,
       31,   36,   37,   38,   39,   40,   31,   41,   31,   31,
       19,    1,   20,    1,   21,    1,   22,   23,   24,   25,

       26,   27,   28,   29,   30,   31,   31,   32,   31,   33,
       34,   35,   31,   36,   37,   38,   39,   40,   31,   41,
       31,   31,    1,    1,    1,   21,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static yyconst flex_int32_t yy_meta[42] =
    {   0,
        1,    1,    2,    1,    1,    1,    3,    1,    4,    4,
        5,    6,    6,    7,    1,    1,    1,    1,    8,    1,
        9,   10,   10,   10,   10,   10,   10,   11,   11,   11,
       11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
       11
    } ;

static yyconst flex_int16_t yy_base[249] =
    {   0,
        0,   40,    0,    0,   40,   41,   43,   44,  651,  650,
       56,    0,  652,  655,   96,  655,  655,  645,  631,  622,
      622,   70,  613,   67,  614,  613,  611,  609,  615,  604,
      610,  132,   99,  655,  655,  655,  655,  655,  625,  655,
      655,  655,  655,  616,  111,  601,  595,  596,  593,  593,
      655,  602,  589,  560,  568,  572,  557,  550,  563,  550,
      547,  546,  530,  540,  522,  534,  521,    0,  655,  100,
      528,  518,  655,  511,   39,  504,  498,  443,  434,  431,
      457,  436,  655,  426,  435,  433,  655,  432,  414,  408,
      411,  419,  421,   91,  408,  407,  416,  395,  378,  368,

      373,  360,  349,  355,  351,  655,  655,  349,    0,  341,
      340,  327,  655,  321,  309,  655,   34,  302,  655,  298,
      296,   91,  105,  133,  280,  314,  287,  137,   94,  163,
      266,  151,  287,  115,  166,  119,  655,  293,  150,  170,
      266,  178,  155,  270,  187,  131,  205,  238,  196,  260,
      181,  208,  210,  251,    0,    0,  266,  223,  253,  235,
      229,  220,  222,  237,  241,  221,  263,  232,  216,  268,
      243,  276,  185,  284,  209,  202,  291,  267,  184,  180,
      157,  154,  253,  293,    0,  297,  313,  299,  302,  317,
      321,  129,  343,  325,  139,  348,  323,  356,   81,  655,

       96,  333,  358,    0,  360,  367,  365,  377,  392,  381,
       28,  389,  397,    0,  387,  399,    0,  408,  419,  417,
      423,  438,  425,  427,    0,  429,  655,  470,  481,  492,
      503,  511,  517,  528,  531,  537,  546,  555,  566,  572,
      580,  586,  596,  607,  613,  621,  627,  632
    } ;

static yyconst flex_int16_t yy_def[249] =
    {   0,
      227,    1,  228,  228,  229,  229,  230,  230,  231,  231,
      227,   11,  227,  227,  227,  227,  227,  227,  232,  227,
      227,  227,  227,  227,  227,  227,  227,  227,  227,  227,
      227,  227,  227,  227,  227,  227,  227,  227,  227,  227,
      227,  227,  227,  233,  227,  227,  227,  227,  227,  227,
      227,  232,  234,  227,  227,  227,  227,  227,  227,  227,
      227,  227,  227,  227,  227,  227,  227,   32,  227,  227,
      227,  227,  227,  233,  227,  234,  234,  227,  227,  227,
      227,  227,  227,  227,  227,  227,  227,  227,  227,  227,
      227,  227,  227,  227,  227,  227,  227,  227,  227,  227,

      227,  227,  227,  227,  227,  227,  227,  227,  235,  227,
      227,  227,  227,  227,  227,  227,  236,  227,  227,  227,
      227,  236,  237,  227,  227,  227,  227,  227,  227,  237,
      130,  238,  227,  227,  227,  227,  227,  239,  227,  130,
      140,  227,  140,  240,  227,  227,  238,  147,  241,  227,
      227,  227,  227,  227,  242,  239,  239,  227,  243,  244,
      227,  227,  140,  140,  147,  165,  227,  165,  245,  227,
      227,  241,  172,  246,  227,  227,  227,  242,  243,  243,
      244,  244,  227,  140,  164,  140,  227,  227,  165,  165,
      172,  191,  227,  191,  247,  227,  227,  246,  198,  227,

      227,  227,  165,  190,  165,  227,  227,  191,  191,  198,
      210,  227,  210,  248,  227,  191,  209,  191,  227,  227,
      210,  210,  227,  210,  222,  210,    0,  227,  227,  227,
      227,  227,  227,  227,  227,  227,  227,  227,  227,  227,
      227,  227,  227,  227,  227,  227,  227,  227
    } ;

static yyconst flex_int16_t yy_nxt[697] =
    {   0,
       14,   15,   16,   14,   17,   14,   14,   14,   14,   14,
       18,   14,   14,   14,   14,   14,   14,   19,   14,   14,
       14,   20,   21,   22,   23,   24,   25,   26,   14,   27,
       14,   28,   29,   14,   30,   14,   31,   14,   14,   14,
       14,   32,   34,   34,   33,   38,   38,  123,   39,   39,
       94,   94,  124,  212,   36,   36,   34,   42,   43,   34,
       34,   34,   34,   42,   34,   34,   34,   34,   34,   34,
       34,   34,   34,   34,   34,   34,   44,   44,   44,   44,
       44,   44,   44,   44,   44,   44,   44,   44,   44,   44,
       44,   44,   44,   44,   44,   44,   44,   45,   56,   59,

       70,   70,   94,   94,  123,  140,  140,   60,   57,  124,
      106,  128,   45,  128,  129,  176,  130,  131,  132,   46,
       47,  200,   48,   71,   71,   49,  151,  152,   72,   72,
      136,  136,   50,   68,   46,   47,   69,   48,  153,  134,
       49,  134,  165,  165,  135,  136,  129,   50,  130,  131,
      154,  158,  174,  159,  193,   46,   47,  145,   48,  145,
      146,   49,  147,  148,  149,  160,  227,  227,   50,  227,
      182,  227,  141,  182,  131,  131,  132,  136,  136,  227,
      133,  140,  140,  180,  162,  153,  162,  180,  142,  163,
      163,  132,  152,  152,  143,  161,  146,  154,  147,  148,

      176,  133,  170,  144,  170,  171,  155,  172,  173,  174,
      133,  227,  154,  227,  166,  123,  148,  148,  149,  152,
      152,  155,  174,  123,  158,  175,  159,  176,  177,  149,
      167,  163,  163,  163,  163,  183,  168,  183,  160,  154,
      184,  184,  132,  227,  227,  169,  167,  133,  164,  164,
      227,  182,  165,  165,  191,  191,  180,  150,  185,  185,
      185,  185,  185,  185,  184,  184,  187,  157,  186,  188,
      176,  188,  150,  149,  189,  189,  149,  171,  150,  172,
      173,  150,  227,  132,  227,  192,  176,  173,  173,  174,
      196,  142,  196,  197,  157,  198,  199,  134,  201,  134,

      132,  193,  151,  152,  184,  184,  133,  194,  227,  227,
      189,  189,  139,  189,  189,  138,  195,  137,  133,  202,
      127,  202,  133,  126,  203,  203,  149,  150,  190,  190,
      227,  125,  191,  191,  210,  210,  227,  227,  204,  204,
      204,  204,  204,  204,  203,  203,  206,  121,  205,  207,
      175,  207,  175,  120,  208,  208,  174,  197,  119,  198,
      199,  175,  227,  118,  227,  211,   87,  199,  199,  203,
      203,  227,  227,  215,  116,  215,  208,  208,  216,  216,
      174,  212,  115,  150,  114,  150,   87,  213,  208,  208,
      227,  113,  210,  210,  112,  220,  214,  220,  216,  216,

      221,  221,  175,  209,  209,   87,  219,  111,  227,  227,
      216,  216,  200,  217,  217,  217,  217,  217,  217,  227,
      227,  200,  200,  218,  175,  223,  110,  223,  221,  221,
      224,  224,  109,  175,  221,  221,  224,  224,  224,  224,
      227,  227,  108,  107,  105,  104,  103,   81,  200,  222,
      222,   81,  200,  102,  200,  101,  100,   99,   98,  225,
      225,  225,  225,  225,  225,   97,   87,   96,   95,  226,
       34,   34,   34,   34,   34,   34,   34,   34,   34,   34,
       34,   35,   35,   35,   35,   35,   35,   35,   35,   35,
       35,   35,   37,   37,   37,   37,   37,   37,   37,   37,

       37,   37,   37,   40,   40,   40,   40,   40,   40,   40,
       40,   40,   40,   40,   52,   52,   52,   77,   52,   52,
       52,   52,   74,   77,   74,   74,   74,   74,   76,   75,
       76,   76,   76,   76,   76,   76,   76,   76,   76,  117,
      117,  117,  122,  122,  122,  122,  122,  122,  133,  133,
       93,  133,  133,   92,   91,  133,  133,  150,  150,   90,
      150,  150,   89,   88,  150,  150,  156,   87,  156,  156,
      156,  156,  156,  156,  156,  156,  156,  164,  164,   86,
       85,  164,  175,  175,   84,  175,  175,   83,   82,  175,
      175,  178,   81,   80,   79,  178,  179,   78,  179,  179,

      179,  179,  179,  179,  179,  179,  179,  181,   77,  181,
      181,  181,  181,  181,  181,  181,  181,  181,  190,  190,
       53,   67,  190,  200,  200,   63,  200,   61,   58,   56,
      200,  200,  209,  209,   75,   73,  209,  222,   67,   66,
       65,  222,   64,   63,   62,   61,   58,   55,   54,   53,
       51,  227,   41,   41,   13,  227,  227,  227,  227,  227,
      227,  227,  227,  227,  227,  227,  227,  227,  227,  227,
      227,  227,  227,  227,  227,  227,  227,  227,  227,  227,
      227,  227,  227,  227,  227,  227,  227,  227,  227,  227,
      227,  227,  227,  227,  227,  227

    } ;

static yyconst flex_int16_t yy_chk[697] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    2,    5,    6,    2,    7,    8,  117,    7,    8,
       75,   75,  117,  211,    5,    6,   11,   11,   11,   11,
       11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
       11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
       11,   11,   11,   11,   11,   11,   11,   11,   11,   11,
       11,   11,   11,   11,   11,   11,   11,   15,   22,   24,

       33,   70,   94,   94,  122,  129,  129,   24,   22,  122,
       94,  123,   45,  123,  123,  201,  123,  123,  123,   15,
       15,  199,   15,   33,   70,   15,  134,  134,   33,   70,
      136,  136,   15,   32,   45,   45,   32,   45,  136,  124,
       45,  124,  146,  146,  124,  124,  128,   45,  128,  128,
      136,  139,  195,  139,  192,   32,   32,  132,   32,  132,
      132,   32,  132,  132,  132,  139,  143,  143,   32,  130,
      182,  130,  130,  181,  130,  130,  130,  135,  135,  140,
      143,  140,  140,  180,  142,  135,  142,  179,  130,  142,
      142,  142,  151,  151,  130,  140,  145,  135,  145,  145,

      151,  140,  149,  130,  149,  149,  135,  149,  149,  149,
      140,  147,  151,  147,  147,  176,  147,  147,  147,  152,
      152,  151,  175,  153,  158,  173,  158,  152,  153,  169,
      147,  162,  162,  163,  163,  161,  147,  161,  158,  152,
      161,  161,  161,  168,  168,  147,  166,  163,  164,  164,
      165,  160,  165,  165,  171,  171,  159,  168,  164,  164,
      164,  164,  164,  164,  183,  183,  165,  157,  164,  167,
      154,  167,  165,  150,  167,  167,  167,  170,  148,  170,
      170,  165,  172,  144,  172,  172,  178,  172,  172,  172,
      174,  141,  174,  174,  138,  174,  174,  177,  178,  177,

      133,  172,  177,  177,  184,  184,  131,  172,  186,  186,
      188,  188,  127,  189,  189,  126,  172,  125,  184,  187,
      121,  187,  186,  120,  187,  187,  187,  189,  190,  190,
      191,  118,  191,  191,  197,  197,  194,  194,  190,  190,
      190,  190,  190,  190,  202,  202,  191,  115,  190,  193,
      194,  193,  191,  114,  193,  193,  193,  196,  112,  196,
      196,  191,  198,  111,  198,  198,  110,  198,  198,  203,
      203,  205,  205,  206,  108,  206,  207,  207,  206,  206,
      206,  198,  105,  203,  104,  205,  103,  198,  208,  208,
      210,  102,  210,  210,  101,  212,  198,  212,  215,  215,

      212,  212,  208,  209,  209,  100,  210,   99,  213,  213,
      216,  216,  210,  209,  209,  209,  209,  209,  209,  218,
      218,  210,  213,  209,  216,  219,   98,  219,  220,  220,
      219,  219,   97,  218,  221,  221,  223,  223,  224,  224,
      226,  226,   96,   95,   93,   92,   91,   90,  221,  222,
      222,   89,  224,   88,  226,   86,   85,   84,   82,  222,
      222,  222,  222,  222,  222,   81,   80,   79,   78,  222,
      228,  228,  228,  228,  228,  228,  228,  228,  228,  228,
      228,  229,  229,  229,  229,  229,  229,  229,  229,  229,
      229,  229,  230,  230,  230,  230,  230,  230,  230,  230,

      230,  230,  230,  231,  231,  231,  231,  231,  231,  231,
      231,  231,  231,  231,  232,  232,  232,   77,  232,  232,
      232,  232,  233,   76,  233,  233,  233,  233,  234,   74,
      234,  234,  234,  234,  234,  234,  234,  234,  234,  235,
      235,  235,  236,  236,  236,  236,  236,  236,  237,  237,
       72,  237,  237,   71,   67,  237,  237,  238,  238,   66,
      238,  238,   65,   64,  238,  238,  239,   63,  239,  239,
      239,  239,  239,  239,  239,  239,  239,  240,  240,   62,
       61,  240,  241,  241,   60,  241,  241,   59,   58,  241,
      241,  242,   57,   56,   55,  242,  243,   54,  243,  243,

      243,  243,  243,  243,  243,  243,  243,  244,   53,  244,
      244,  244,  244,  244,  244,  244,  244,  244,  245,  245,
       52,   50,  245,  246,  246,   49,  246,   48,   47,   46,
      246,  246,  247,  247,   44,   39,  247,  248,   31,   30,
       29,  248,   28,   27,   26,   25,   23,   21,   20,   19,
       18,   13,   10,    9,  227,  227,  227,  227,  227,  227,
      227,  227,  227,  227,  227,  227,  227,  227,  227,  227,
      227,  227,  227,  227,  227,  227,  227,  227,  227,  227,
      227,  227,  227,  227,  227,  227,  227,  227,  227,  227,
      227,  227,  227,  227,  227,  227

    } ;

static yy_state_type yy_last_accepting_state;
static char *yy_last_accepting_cpos;

extern int calc_flex_debug;
int calc_flex_debug = 0;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
#ifndef YYLMAX
#define YYLMAX 8192
#endif

char calctext[YYLMAX];
char *calctext_ptr;
#line 1 "calc.l"
#line 5 "calc.l"
#include <ctype.h>
#include <funtoolsP.h>
#include <macro.h>
#include <calc.h>
#include <tabcalc_c.h>

/* define the types of event records we have to handle */
#define REC_CUR		1
#define REC_PREV	2
#define REC_NEXT	4

/* misc */
#define MAX_INCLUDE_DEPTH  100
#define SEGINC            1024
#define MAXCODE 	     4

/* symbols for each column we encounter */
typedef struct calccoltab {
  struct calccoltab *next;
  char *name;
  char *sname;
  char *cname;
  char *tdim;
  int type;
  int n;
  double tlmin;
  double tlmax;
  double binsiz;
  double tscale;
  double tzero;
  int scaled;
  int exists;
  int settype;
} *CalcCols, CalcColRec;

static YY_BUFFER_STATE include_stack[MAX_INCLUDE_DEPTH];
static int include_stack_ptr = 0;
static Fun ifun;
static CalcCols columns;
static int laststart;

static char *prog;
static char *command;
static char *autodeclare;
static char *mbuf;
static int mlen;
static int n;
static int var;
static int rectype;
static int args;
static int doreplace=0;

static char *code[MAXCODE];
static char *expr;
static char **strptr=&expr;
static int ncode[MAXCODE];
static int nexpr;
static int xnl;
static int *lenptr=&nexpr;

static void setn _PRx((int newn));
static char *_CalcMake _PRx((void));
static int _CalcColProc _PRx((char *s, int expl));
static int _CalcColName _PRx((char *sname));
static CalcCols _CalcColNew _PRx((Fun fun, 
                                  char *name, char *sname, char *cname, 
			          int type, int n, 
				  double tlmin, double tlmax, double binsiz,
				  double tscale, double tzero, int scaled,
				  int exists, int settype));
static CalcCols _CalcColLookup _PRx((char *cname));

int _calcerror _PRx((char *msg));





#line 790 "lex.calc.c"

#define INITIAL 0
#define INCLUDE 1
#define VAR 2
#define COM 3
#define XCOM 4
#define ECOL 5

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
#include <unistd.h>
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

static int yy_init_globals (void );

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int calclex_destroy (void );

int calcget_debug (void );

void calcset_debug (int debug_flag  );

YY_EXTRA_TYPE calcget_extra (void );

void calcset_extra (YY_EXTRA_TYPE user_defined  );

FILE *calcget_in (void );

void calcset_in  (FILE * in_str  );

FILE *calcget_out (void );

void calcset_out  (FILE * out_str  );

int calcget_leng (void );

char *calcget_text (void );

int calcget_lineno (void );

void calcset_lineno (int line_number  );

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int calcwrap (void );
#else
extern int calcwrap (void );
#endif
#endif

    static void yyunput (int c,char *buf_ptr  );
    
#ifndef calctext_ptr
static void yy_flex_strncpy (char *,yyconst char *,int );
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * );
#endif

#ifndef YY_NO_INPUT

#ifdef __cplusplus
static int yyinput (void );
#else
static int input (void );
#endif

#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#define YY_READ_BUF_SIZE 8192
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO (void) fwrite( calctext, calcleng, 1, calcout )
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( YY_CURRENT_BUFFER_LVALUE->yy_is_interactive ) \
		{ \
		int c = '*'; \
		size_t n; \
		for ( n = 0; n < max_size && \
			     (c = getc( calcin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( calcin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = fread(buf, 1, max_size, calcin))==0 && ferror(calcin)) \
			{ \
			if( errno != EINTR) \
				{ \
				YY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(calcin); \
			} \
		}\
\

#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg )
#endif

/* end tables serialization structures and prototypes */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1

extern int calclex (void);

#define YY_DECL int calclex (void)
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after calctext and calcleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

#define YY_RULE_SETUP \
	if ( calcleng > 0 ) \
		YY_CURRENT_BUFFER_LVALUE->yy_at_bol = \
				(calctext[calcleng - 1] == '\n'); \
	YY_USER_ACTION

/** The main scanner function which does all the work.
 */
YY_DECL
{
	register yy_state_type yy_current_state;
	register char *yy_cp, *yy_bp;
	register int yy_act;
    
#line 109 "calc.l"


#line 983 "lex.calc.c"

	if ( !(yy_init) )
		{
		(yy_init) = 1;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! (yy_start) )
			(yy_start) = 1;	/* first start state */

		if ( ! calcin )
			calcin = stdin;

		if ( ! calcout )
			calcout = stdout;

		if ( ! YY_CURRENT_BUFFER ) {
			calcensure_buffer_stack ();
			YY_CURRENT_BUFFER_LVALUE =
				calc_create_buffer(calcin,YY_BUF_SIZE );
		}

		calc_load_buffer_state( );
		}

	while ( 1 )		/* loops until end-of-file is reached */
		{
		yy_cp = (yy_c_buf_p);

		/* Support of calctext. */
		*yy_cp = (yy_hold_char);

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		yy_current_state = (yy_start);
		yy_current_state += YY_AT_BOL();
yy_match:
		do
			{
			register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
			if ( yy_accept[yy_current_state] )
				{
				(yy_last_accepting_state) = yy_current_state;
				(yy_last_accepting_cpos) = yy_cp;
				}
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 228 )
					yy_c = yy_meta[(unsigned int) yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
			++yy_cp;
			}
		while ( yy_base[yy_current_state] != 655 );

yy_find_action:
		yy_act = yy_accept[yy_current_state];
		if ( yy_act == 0 )
			{ /* have to back up */
			yy_cp = (yy_last_accepting_cpos);
			yy_current_state = (yy_last_accepting_state);
			yy_act = yy_accept[yy_current_state];
			}

		YY_DO_BEFORE_ACTION;

do_action:	/* This label is used only to access EOF actions. */

		switch ( yy_act )
	{ /* beginning of action switch */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*yy_cp = (yy_hold_char);
			yy_cp = (yy_last_accepting_cpos);
			yy_current_state = (yy_last_accepting_state);
			goto yy_find_action;

case 1:
YY_RULE_SETUP
#line 111 "calc.l"
{ setn(0);  var = 0; }
	YY_BREAK
case 2:
YY_RULE_SETUP
#line 112 "calc.l"
{ setn(1);  var = 0; }
	YY_BREAK
case 3:
YY_RULE_SETUP
#line 113 "calc.l"
{ setn(2);  var = 0; }
	YY_BREAK
case 4:
YY_RULE_SETUP
#line 114 "calc.l"
{ setn(3);  var = 0; }
	YY_BREAK
case 5:
YY_RULE_SETUP
#line 115 "calc.l"
{ setn(-1); var = 0; }
	YY_BREAK
case 6:
YY_RULE_SETUP
#line 117 "calc.l"
{
		char *s;
		if ( include_stack_ptr >= MAX_INCLUDE_DEPTH )
		  _calcerror("include files are nested too deeply");
		if( !(s = (char *)FileContents(calctext+1, 0, NULL)) )
		  _calcerror("can't access include file");
		else {
		  include_stack[include_stack_ptr++] = YY_CURRENT_BUFFER;
		  BEGIN INITIAL; calc_scan_string(s); BEGIN INITIAL;
		  if( s ) xfree(s);
		}
}
	YY_BREAK
case 7:
YY_RULE_SETUP
#line 130 "calc.l"
{ 
		if( var ){ BEGIN VAR; setn(1); }
		_CalcCat(calctext, strptr, lenptr);
}
	YY_BREAK
case 8:
YY_RULE_SETUP
#line 134 "calc.l"
{
		_CalcCat(calctext, strptr, lenptr);
		_CalcCat("\n", strptr, lenptr);
		if( var ){ BEGIN INITIAL; setn(-1); }
}
	YY_BREAK
case 9:
YY_RULE_SETUP
#line 139 "calc.l"
{ _CalcCat(calctext, strptr, lenptr); }
	YY_BREAK
case 10:
YY_RULE_SETUP
#line 141 "calc.l"
{ _CalcCat(calctext, strptr, lenptr); }
	YY_BREAK
case 11:
YY_RULE_SETUP
#line 143 "calc.l"
{ _CalcCat(calctext, strptr, lenptr); }
	YY_BREAK
case 12:
YY_RULE_SETUP
#line 145 "calc.l"
{ laststart = YY_START; xnl=0; BEGIN XCOM; }
	YY_BREAK
case 13:
YY_RULE_SETUP
#line 146 "calc.l"
{ laststart = YY_START; xnl=1; BEGIN XCOM; }
	YY_BREAK
case 14:
/* rule 14 can match eol */
YY_RULE_SETUP
#line 147 "calc.l"
{ BEGIN laststart; if( xnl ) unput('\n'); }
	YY_BREAK
case 15:
YY_RULE_SETUP
#line 148 "calc.l"
{ /* ignore comments up to eol */ ; }
	YY_BREAK
case 16:
YY_RULE_SETUP
#line 150 "calc.l"
{
	         _CalcCat(calctext, strptr, lenptr);
		 laststart = YY_START; BEGIN COM;
}
	YY_BREAK
case 17:
YY_RULE_SETUP
#line 154 "calc.l"
{ _CalcCat(calctext, strptr, lenptr); BEGIN laststart; }
	YY_BREAK
case 18:
/* rule 18 can match eol */
YY_RULE_SETUP
#line 155 "calc.l"
{ _CalcCat(calctext, strptr, lenptr); }
	YY_BREAK
case 19:
YY_RULE_SETUP
#line 156 "calc.l"
{ _CalcCat(calctext, strptr, lenptr); }
	YY_BREAK
case 20:
YY_RULE_SETUP
#line 158 "calc.l"
{ _CalcColProc(calctext, 0); var=0;}
	YY_BREAK
case 21:
YY_RULE_SETUP
#line 160 "calc.l"
{  laststart = YY_START; BEGIN ECOL; }
	YY_BREAK
case 22:
YY_RULE_SETUP
#line 161 "calc.l"
{ _CalcColProc(calctext, 1); var=0; }
	YY_BREAK
case 23:
YY_RULE_SETUP
#line 162 "calc.l"
{ ; }
	YY_BREAK
case 24:
/* rule 24 can match eol */
YY_RULE_SETUP
#line 163 "calc.l"
{ BEGIN laststart; }
	YY_BREAK
case 25:
/* rule 25 can match eol */
YY_RULE_SETUP
#line 165 "calc.l"
{ _CalcCat(calctext, strptr, lenptr);}
	YY_BREAK
case 26:
YY_RULE_SETUP
#line 167 "calc.l"
{ _CalcCat(calctext, strptr, lenptr);}
	YY_BREAK
case 27:
YY_RULE_SETUP
#line 169 "calc.l"
{ _CalcCat(calctext, strptr, lenptr); var=0;}
	YY_BREAK
case YY_STATE_EOF(INITIAL):
case YY_STATE_EOF(INCLUDE):
case YY_STATE_EOF(VAR):
case YY_STATE_EOF(COM):
case YY_STATE_EOF(XCOM):
case YY_STATE_EOF(ECOL):
#line 171 "calc.l"
{
		if ( --include_stack_ptr < 0 ){
		  prog = _CalcMake();
		  calc_delete_buffer(YY_CURRENT_BUFFER );
                  yyterminate();
		} else {
		  calc_delete_buffer(YY_CURRENT_BUFFER );
		  calc_switch_to_buffer(include_stack[include_stack_ptr] );
		}
}
	YY_BREAK
case 28:
YY_RULE_SETUP
#line 182 "calc.l"
ECHO;
	YY_BREAK
#line 1250 "lex.calc.c"

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - (calctext_ptr)) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = (yy_hold_char);
		YY_RESTORE_YY_MORE_OFFSET

		if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed calcin at a new source and called
			 * calclex().  If so, then we have to assure
			 * consistency between YY_CURRENT_BUFFER and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
			YY_CURRENT_BUFFER_LVALUE->yy_input_file = calcin;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( (yy_c_buf_p) <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			(yy_c_buf_p) = (calctext_ptr) + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state(  );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state );

			yy_bp = (calctext_ptr) + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++(yy_c_buf_p);
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
				yy_cp = (yy_c_buf_p);
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer(  ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				(yy_did_buffer_switch_on_eof) = 0;

				if ( calcwrap( ) )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * calctext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					(yy_c_buf_p) = (calctext_ptr) + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				(yy_c_buf_p) =
					(calctext_ptr) + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (calctext_ptr) + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				(yy_c_buf_p) =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)];

				yy_current_state = yy_get_previous_state(  );

				yy_cp = (yy_c_buf_p);
				yy_bp = (calctext_ptr) + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
} /* end of calclex */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */
static int yy_get_next_buffer (void)
{
    	register char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
	register char *source = (calctext_ptr);
	register int number_to_move, i;
	int ret_val;

	if ( (yy_c_buf_p) > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( (yy_c_buf_p) - (calctext_ptr) - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) ((yy_c_buf_p) - (calctext_ptr)) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars) = 0;

	else
		{
			int num_to_read =
			YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = YY_CURRENT_BUFFER;

			int yy_c_buf_p_offset =
				(int) ((yy_c_buf_p) - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
				{
				int new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					calcrealloc((void *) b->yy_ch_buf,b->yy_buf_size + 2  );
				}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = 0;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			(yy_c_buf_p) = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = YY_CURRENT_BUFFER_LVALUE->yy_buf_size -
						number_to_move - 1;

			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
			(yy_n_chars), num_to_read );

		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	if ( (yy_n_chars) == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			calcrestart(calcin  );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	(yy_n_chars) += number_to_move;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] = YY_END_OF_BUFFER_CHAR;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars) + 1] = YY_END_OF_BUFFER_CHAR;

	(calctext_ptr) = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

	return ret_val;
}

/* yy_get_previous_state - get the state just before the EOB char was reached */

    static yy_state_type yy_get_previous_state (void)
{
	register yy_state_type yy_current_state;
	register char *yy_cp;
    
	yy_current_state = (yy_start);
	yy_current_state += YY_AT_BOL();

	for ( yy_cp = (calctext_ptr) + YY_MORE_ADJ; yy_cp < (yy_c_buf_p); ++yy_cp )
		{
		register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		if ( yy_accept[yy_current_state] )
			{
			(yy_last_accepting_state) = yy_current_state;
			(yy_last_accepting_cpos) = yy_cp;
			}
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 228 )
				yy_c = yy_meta[(unsigned int) yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
		}

	return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */
    static yy_state_type yy_try_NUL_trans  (yy_state_type yy_current_state )
{
	register int yy_is_jam;
    	register char *yy_cp = (yy_c_buf_p);

	register YY_CHAR yy_c = 1;
	if ( yy_accept[yy_current_state] )
		{
		(yy_last_accepting_state) = yy_current_state;
		(yy_last_accepting_cpos) = yy_cp;
		}
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 228 )
			yy_c = yy_meta[(unsigned int) yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
	yy_is_jam = (yy_current_state == 227);

	return yy_is_jam ? 0 : yy_current_state;
}

    static void yyunput (int c, register char * yy_bp )
{
	register char *yy_cp;
    
    yy_cp = (yy_c_buf_p);

	/* undo effects of setting up calctext */
	*yy_cp = (yy_hold_char);

	if ( yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2 )
		{ /* need to shift things up to make room */
		/* +2 for EOB chars. */
		register int number_to_move = (yy_n_chars) + 2;
		register char *dest = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[
					YY_CURRENT_BUFFER_LVALUE->yy_buf_size + 2];
		register char *source =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move];

		while ( source > YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			*--dest = *--source;

		yy_cp += (int) (dest - source);
		yy_bp += (int) (dest - source);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars =
			(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_buf_size;

		if ( yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2 )
			YY_FATAL_ERROR( "flex scanner push-back overflow" );
		}

	*--yy_cp = (char) c;

	(calctext_ptr) = yy_bp;
	(yy_hold_char) = *yy_cp;
	(yy_c_buf_p) = yy_cp;
}

#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int yyinput (void)
#else
    static int input  (void)
#endif

{
	int c;
    
	*(yy_c_buf_p) = (yy_hold_char);

	if ( *(yy_c_buf_p) == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( (yy_c_buf_p) < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[(yy_n_chars)] )
			/* This was really a NUL. */
			*(yy_c_buf_p) = '\0';

		else
			{ /* need more input */
			int offset = (yy_c_buf_p) - (calctext_ptr);
			++(yy_c_buf_p);

			switch ( yy_get_next_buffer(  ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					calcrestart(calcin );

					/*FALLTHROUGH*/

				case EOB_ACT_END_OF_FILE:
					{
					if ( calcwrap( ) )
						return EOF;

					if ( ! (yy_did_buffer_switch_on_eof) )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					(yy_c_buf_p) = (calctext_ptr) + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) (yy_c_buf_p);	/* cast for 8-bit char's */
	*(yy_c_buf_p) = '\0';	/* preserve calctext */
	(yy_hold_char) = *++(yy_c_buf_p);

	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = (c == '\n');

	return c;
}
#endif	/* ifndef YY_NO_INPUT */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * 
 * @note This function does not reset the start condition to @c INITIAL .
 */
    void calcrestart  (FILE * input_file )
{
    
	if ( ! YY_CURRENT_BUFFER ){
        calcensure_buffer_stack ();
		YY_CURRENT_BUFFER_LVALUE =
            calc_create_buffer(calcin,YY_BUF_SIZE );
	}

	calc_init_buffer(YY_CURRENT_BUFFER,input_file );
	calc_load_buffer_state( );
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * 
 */
    void calc_switch_to_buffer  (YY_BUFFER_STATE  new_buffer )
{
    
	/* TODO. We should be able to replace this entire function body
	 * with
	 *		calcpop_buffer_state();
	 *		calcpush_buffer_state(new_buffer);
     */
	calcensure_buffer_stack ();
	if ( YY_CURRENT_BUFFER == new_buffer )
		return;

	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	YY_CURRENT_BUFFER_LVALUE = new_buffer;
	calc_load_buffer_state( );

	/* We don't actually know whether we did this switch during
	 * EOF (calcwrap()) processing, but the only time this flag
	 * is looked at is after calcwrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	(yy_did_buffer_switch_on_eof) = 1;
}

static void calc_load_buffer_state  (void)
{
    	(yy_n_chars) = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
	(calctext_ptr) = (yy_c_buf_p) = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
	calcin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
	(yy_hold_char) = *(yy_c_buf_p);
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * 
 * @return the allocated buffer state.
 */
    YY_BUFFER_STATE calc_create_buffer  (FILE * file, int  size )
{
	YY_BUFFER_STATE b;
    
	b = (YY_BUFFER_STATE) calcalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in calc_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) calcalloc(b->yy_buf_size + 2  );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in calc_create_buffer()" );

	b->yy_is_our_buffer = 1;

	calc_init_buffer(b,file );

	return b;
}

/** Destroy the buffer.
 * @param b a buffer created with calc_create_buffer()
 * 
 */
    void calc_delete_buffer (YY_BUFFER_STATE  b )
{
    
	if ( ! b )
		return;

	if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
		YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		calcfree((void *) b->yy_ch_buf  );

	calcfree((void *) b  );
}

#ifndef __cplusplus
extern int isatty (int );
#endif /* __cplusplus */
    
/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a calcrestart() or at EOF.
 */
    static void calc_init_buffer  (YY_BUFFER_STATE  b, FILE * file )

{
	int oerrno = errno;
    
	calc_flush_buffer(b );

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

    /* If b is the current buffer, then calc_init_buffer was _probably_
     * called from calcrestart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

        b->yy_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
    
	errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * 
 */
    void calc_flush_buffer (YY_BUFFER_STATE  b )
{
    	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == YY_CURRENT_BUFFER )
		calc_load_buffer_state( );
}

/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  
 */
void calcpush_buffer_state (YY_BUFFER_STATE new_buffer )
{
    	if (new_buffer == NULL)
		return;

	calcensure_buffer_stack();

	/* This block is copied from calc_switch_to_buffer. */
	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*(yy_c_buf_p) = (yy_hold_char);
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = (yy_c_buf_p);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = (yy_n_chars);
		}

	/* Only push if top exists. Otherwise, replace top. */
	if (YY_CURRENT_BUFFER)
		(yy_buffer_stack_top)++;
	YY_CURRENT_BUFFER_LVALUE = new_buffer;

	/* copied from calc_switch_to_buffer. */
	calc_load_buffer_state( );
	(yy_did_buffer_switch_on_eof) = 1;
}

/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  
 */
void calcpop_buffer_state (void)
{
    	if (!YY_CURRENT_BUFFER)
		return;

	calc_delete_buffer(YY_CURRENT_BUFFER );
	YY_CURRENT_BUFFER_LVALUE = NULL;
	if ((yy_buffer_stack_top) > 0)
		--(yy_buffer_stack_top);

	if (YY_CURRENT_BUFFER) {
		calc_load_buffer_state( );
		(yy_did_buffer_switch_on_eof) = 1;
	}
}

/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
static void calcensure_buffer_stack (void)
{
	int num_to_alloc;
    
	if (!(yy_buffer_stack)) {

		/* First allocation is just for 2 elements, since we don't know if this
		 * scanner will even need a stack. We use 2 instead of 1 to avoid an
		 * immediate realloc on the next call.
         */
		num_to_alloc = 1;
		(yy_buffer_stack) = (struct yy_buffer_state**)calcalloc
								(num_to_alloc * sizeof(struct yy_buffer_state*)
								);
		
		memset((yy_buffer_stack), 0, num_to_alloc * sizeof(struct yy_buffer_state*));
				
		(yy_buffer_stack_max) = num_to_alloc;
		(yy_buffer_stack_top) = 0;
		return;
	}

	if ((yy_buffer_stack_top) >= ((yy_buffer_stack_max)) - 1){

		/* Increase the buffer to prepare for a possible push. */
		int grow_size = 8 /* arbitrary grow size */;

		num_to_alloc = (yy_buffer_stack_max) + grow_size;
		(yy_buffer_stack) = (struct yy_buffer_state**)calcrealloc
								((yy_buffer_stack),
								num_to_alloc * sizeof(struct yy_buffer_state*)
								);

		/* zero only the new slots.*/
		memset((yy_buffer_stack) + (yy_buffer_stack_max), 0, grow_size * sizeof(struct yy_buffer_state*));
		(yy_buffer_stack_max) = num_to_alloc;
	}
}

/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * 
 * @return the newly allocated buffer state object. 
 */
YY_BUFFER_STATE calc_scan_buffer  (char * base, yy_size_t  size )
{
	YY_BUFFER_STATE b;
    
	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return 0;

	b = (YY_BUFFER_STATE) calcalloc(sizeof( struct yy_buffer_state )  );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in calc_scan_buffer()" );

	b->yy_buf_size = size - 2;	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = 0;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	calc_switch_to_buffer(b  );

	return b;
}

/** Setup the input buffer state to scan a string. The next call to calclex() will
 * scan from a @e copy of @a str.
 * @param str a NUL-terminated string to scan
 * 
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       calc_scan_bytes() instead.
 */
YY_BUFFER_STATE calc_scan_string (yyconst char * yystr )
{
    
	return calc_scan_bytes(yystr,strlen(yystr) );
}

/** Setup the input buffer state to scan the given bytes. The next call to calclex() will
 * scan from a @e copy of @a bytes.
 * @param bytes the byte buffer to scan
 * @param len the number of bytes in the buffer pointed to by @a bytes.
 * 
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE calc_scan_bytes  (yyconst char * yybytes, int  _yybytes_len )
{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	int i;
    
	/* Get memory for full buffer, including space for trailing EOB's. */
	n = _yybytes_len + 2;
	buf = (char *) calcalloc(n  );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in calc_scan_bytes()" );

	for ( i = 0; i < _yybytes_len; ++i )
		buf[i] = yybytes[i];

	buf[_yybytes_len] = buf[_yybytes_len+1] = YY_END_OF_BUFFER_CHAR;

	b = calc_scan_buffer(buf,n );
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in calc_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
}

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

static void yy_fatal_error (yyconst char* msg )
{
    	(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
}

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up calctext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		calctext[calcleng] = (yy_hold_char); \
		(yy_c_buf_p) = calctext + yyless_macro_arg; \
		(yy_hold_char) = *(yy_c_buf_p); \
		*(yy_c_buf_p) = '\0'; \
		calcleng = yyless_macro_arg; \
		} \
	while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/** Get the current line number.
 * 
 */
int calcget_lineno  (void)
{
        
    return calclineno;
}

/** Get the input stream.
 * 
 */
FILE *calcget_in  (void)
{
        return calcin;
}

/** Get the output stream.
 * 
 */
FILE *calcget_out  (void)
{
        return calcout;
}

/** Get the length of the current token.
 * 
 */
int calcget_leng  (void)
{
        return calcleng;
}

/** Get the current token.
 * 
 */

char *calcget_text  (void)
{
        return calctext;
}

/** Set the current line number.
 * @param line_number
 * 
 */
void calcset_lineno (int  line_number )
{
    
    calclineno = line_number;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param in_str A readable stream.
 * 
 * @see calc_switch_to_buffer
 */
void calcset_in (FILE *  in_str )
{
        calcin = in_str ;
}

void calcset_out (FILE *  out_str )
{
        calcout = out_str ;
}

int calcget_debug  (void)
{
        return calc_flex_debug;
}

void calcset_debug (int  bdebug )
{
        calc_flex_debug = bdebug ;
}

static int yy_init_globals (void)
{
        /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from calclex_destroy(), so don't allocate here.
     */

    (yy_buffer_stack) = 0;
    (yy_buffer_stack_top) = 0;
    (yy_buffer_stack_max) = 0;
    (yy_c_buf_p) = (char *) 0;
    (yy_init) = 0;
    (yy_start) = 0;

/* Defined in main.c */
#ifdef YY_STDINIT
    calcin = stdin;
    calcout = stdout;
#else
    calcin = (FILE *) 0;
    calcout = (FILE *) 0;
#endif

    /* For future reference: Set errno on error, since we are called by
     * calclex_init()
     */
    return 0;
}

/* calclex_destroy is for both reentrant and non-reentrant scanners. */
int calclex_destroy  (void)
{
    
    /* Pop the buffer stack, destroying each element. */
	while(YY_CURRENT_BUFFER){
		calc_delete_buffer(YY_CURRENT_BUFFER  );
		YY_CURRENT_BUFFER_LVALUE = NULL;
		calcpop_buffer_state();
	}

	/* Destroy the stack itself. */
	calcfree((yy_buffer_stack) );
	(yy_buffer_stack) = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * calclex() is called, initialization will occur. */
    yy_init_globals( );

    return 0;
}

/*
 * Internal utility routines.
 */

#ifndef calctext_ptr
static void yy_flex_strncpy (char* s1, yyconst char * s2, int n )
{
	register int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * s )
{
	register int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif

void *calcalloc (yy_size_t  size )
{
	return (void *) malloc( size );
}

void *calcrealloc  (void * ptr, yy_size_t  size )
{
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return (void *) realloc( (char *) ptr, size );
}

void calcfree (void * ptr )
{
	free( (char *) ptr );	/* see calcrealloc() for (char *) cast */
}

#define YYTABLES_NAME "yytables"

#line 182 "calc.l"



/*
 *
 * Private Routines 
 *
 *
 */

/* setn -- set pointer to where expression is stored */
#ifdef YY_USE_PROTOS
static void setn(int newn)
#else
static void setn(newn)
    int newn;
#endif
{
  if( newn >=0 ){
    n = newn;
    strptr = &code[n]; lenptr = &ncode[n];
  }
  else{
    strptr = &expr;    lenptr = &nexpr;
  }
}

/*
 *
 * _CalcDatatype -- return C data type as a string
 *
 */
#ifdef YY_USE_PROTOS
static char *
_CalcDatatype(int type)
#else
static char *_CalcDatatype(type)
	int type;
#endif
{
    switch ( type ) {
     case 'A':		return "char";
     case 'X':		return "char";
     case 'B':		return "unsigned char";
     case 'I':		return "short";
     case 'U':		return "unsigned short";
     case 'J':		return "int";
     case 'K':		return "long long";
     case 'V':		return "unsigned int";
     case 'L':		return "char";
     case 'E':		return "float";
     case 'D':		return "double";
    }
    _calcerror("unknown/illegal data type for new column");
    return "???";
}

/*
 *
 * _CalcColName -- see if sname is a recognized column name
 *
 */
#ifdef YY_USE_PROTOS
static int
_CalcColName(char *sname)
#else
static int _CalcColName(sname)
     char *sname;
#endif
{
  if( !strcasecmp(sname, "prev") ){
    strcpy(sname, "prev");
    rectype |= REC_PREV;
    return REC_PREV;
  }
  if( !strcasecmp(sname, "cur") ){
    strcpy(sname, "cur");
    rectype |= REC_CUR;
    return REC_CUR;
  }
  if( !strcasecmp(sname, "next") ){
    strcpy(sname, "next");
    rectype |= REC_NEXT;
    return REC_NEXT;
  }
  else{
    return 0;
  }
}

/*
 *
 * _CalcCB -- macro expansion callback
 *
 */
#ifdef YY_USE_PROTOS
static char *
_CalcCB(char *name, void *client_data)
#else
static char *_CalcCB(name, client_data)
	char *name;
	void *client_data;
#endif
{
  CalcCols cur;  
  char tbuf[SZ_LINE];
  char tbuf2[SZ_LINE];
  char tbuf3[SZ_LINE];
  int i, got, ip;
  
  /* avoid -W unused parameter warning */
  if( 0 ) client_data = client_data;

  /* start at beginning of macro buffer */
  /* allocate the macro buffer */
  if( mbuf ) xfree(mbuf);
  mbuf = NULL;
  mlen = SZ_LINE;

  if( !strcmp(name, "MEMBERS") ){
    if( !columns ){
      _CalcCat("int dummy;\n", &mbuf, &mlen);
      return mbuf;
    }    
    for(cur=columns; cur!=NULL; cur=cur->next){
      snprintf(tbuf, SZ_LINE, "%s %s", _CalcDatatype(cur->type), cur->cname);
      _CalcCat(tbuf, &mbuf, &mlen);
      if( cur->n > 1 ){
        /* if there is no tdim, just make it a simple array */
        /* (also, do the simple thing for bit-fields) */
        if( (cur->tdim == NULL) || (cur->type == 'X') ){
          switch(cur->type){
          case 'X':
            i = (cur->n+7)/8;
            break;
          default:
            i = cur->n;
            break;
          }
          snprintf(tbuf, SZ_LINE, "[%d]", i);
        }
        /* tdim exists, so use it to declare a multidimensional array */
        else{
          newdtable(",()");
          /* I hear tdim is in Fortran order, so reverse it for C order */
          for(*tbuf = '\0', ip=0, got=0; word(cur->tdim, tbuf3, &ip); got++){
            snprintf(tbuf2, SZ_LINE, "[%s]", tbuf3);
            if( *tbuf )
              strcat(tbuf2, tbuf);
            strcpy(tbuf, tbuf2);
          }          
          freedtable();
          /* but if we got nothing out of it, go back to simplicity */
          if( !got )
            snprintf(tbuf, SZ_LINE, "[%d]", cur->n);
        }
        _CalcCat(tbuf, &mbuf, &mlen);
      }
      _CalcCat(";", &mbuf, &mlen);
      if( cur->next )
        _CalcCat("\n", &mbuf, &mlen);
    }
    return mbuf;
  }
  else if( !strcmp(name, "SELECT") ){
    if( !columns ){
      return "";
    }    
    for(cur=columns; cur!=NULL; cur=cur->next){
      *tbuf2 = '\0';
      if( (cur->tlmin != 0.0) || (cur->tlmax != 0.0) ){
        snprintf(tbuf2, SZ_LINE, "%f:%f", cur->tlmin, cur->tlmax);
        if( cur->binsiz != 1.0 ){
	  snprintf(tbuf3, SZ_LINE, ":%f", cur->binsiz);
	  strncat(tbuf2, tbuf3, SZ_LINE);
	}
        if( (cur->tscale != 1.0) || (cur->tzero != 0.0) ){
	  snprintf(tbuf3, SZ_LINE, ";%f", cur->tscale);
	  strncat(tbuf2, tbuf3, SZ_LINE);
	  snprintf(tbuf3, SZ_LINE, ":%f", cur->tzero);
	  strncat(tbuf2, tbuf3, SZ_LINE);
	}
      }
      if( *tbuf2 ){
        snprintf(tbuf, SZ_LINE, "\"%s\", \"%d%c:%s\", ",
		       cur->cname, cur->n, cur->type, tbuf2);
      }
      else{
        snprintf(tbuf, SZ_LINE, "\"%s\", \"%d%c\", ",
		       cur->cname, cur->n, cur->type);
      }
      _CalcCat(tbuf, &mbuf, &mlen);
      if( cur->exists )
        _CalcCat("\"rw\", ", &mbuf, &mlen);
      else
        _CalcCat("\"w\", ", &mbuf, &mlen);
      snprintf(tbuf, SZ_LINE, "FUN_OFFSET(Row, %s),", cur->cname);
      _CalcCat(tbuf, &mbuf, &mlen);
      if( cur->next )
        _CalcCat("\n", &mbuf, &mlen);
    }
    return mbuf;
  }
  else if( !strcmp(name, "RECTYPE") ){
    snprintf(tbuf, SZ_LINE, "%d", rectype);
    _CalcCat(tbuf, &mbuf, &mlen);
    return mbuf;
  }
  else if( !strcmp(name, "EXPR") ){
    if( expr && *expr )
      return(expr);
    else
      return "";
  }
  else if( !strcmp(name, "COMMAND") ){
    if( command && *command )
      return(command);
    else
      return "<build command not available>";
  }
  else if( !strcmp(name, "ARGS") ){
    if( args == 1 )
      return "1";
    else
      return "2";
  }
  else if( !strcmp(name, "AUTO") ){
    if( autodeclare && *autodeclare )
      return autodeclare;
    else
      return "";
  }
  else if( !strcmp(name, "GLOBAL") ){
    if( code[0] && *code[0] )
      return(code[0]);
    else
      return "";
  }
  else if( !strcmp(name, "LOCAL") ){
    if( code[1] && *code[1] )
      return(code[1]);
    else
      return "";
  }
  else if( !strcmp(name, "BEFORE") ){
    if( code[2] && *code[2] )
      return(code[2]);
    else
      return "";
  }
  else if( !strcmp(name, "AFTER") ){
    if( code[3] && *code[3] )
      return(code[3]);
    else
      return "";
  }
  else if( !strcmp(name, "MERGE") ){
    if( doreplace )
      strncpy(tbuf, "replace", SZ_LINE);
    else
      strncpy(tbuf, "update", SZ_LINE);
    _CalcCat(tbuf, &mbuf, &mlen);
    return mbuf;
  }
  else{
    _CalcCat("$", &mbuf, &mlen);
    _CalcCat(name, &mbuf, &mlen);
    return mbuf;
  }
}

/*
 *
 * _CalcMake -- create the string containing the program by macro expansion
 *
 */
#ifdef YY_USE_PROTOS
static char *
_CalcMake(void)
#else
static char *_CalcMake()
#endif
{
  char *s;

  /* make sure we have something */
  if( !expr || !*expr )
    expr = xstrdup(";");
  s = xstrdup(expr);
  nowhite(s, expr);
  if( s ) xfree(s);
  /* add final ';' is necessary (but not if there is any compund statement) */
  if( !strchr(expr,';') && !strchr(expr,'{') )
    _CalcCat(";", &expr, &nexpr);  
  /* expand program body to add specifics of the expression */
  s = ExpandMacro(TABCALC_C, NULL, NULL, 0, _CalcCB, NULL);
  return s;
}

/* look up a column entry, add if not present */
#ifdef YY_USE_PROTOS
CalcCols
_CalcColLookup(char *cname)
#else
CalcCols
_CalcColLookup(cname)
	char *cname;
#endif
{
  CalcCols cur;  
  for(cur=columns; cur!=NULL; cur=cur->next){
    if( !strcasecmp(cname, cur->cname) )
      return cur;
  }
  return NULL;
}

/* look up a symbol table entry, add if not present */
#ifdef YY_USE_PROTOS
CalcCols
_CalcColNew(Fun fun, char *name, char *sname, char *cname, int type, int n,
            double tlmin, double tlmax, double binsiz, 
	    double tscale, double tzero, int scaled,
	    int exists, int settype)
#else
CalcCols
_CalcColNew(fun, name, sname,
            cname, type, n, tlmin, tlmax, binsiz, 
	    tscale, tzero, scaled,
	    exists, settype)
        Fun fun;
	char *name, *sname, *cname;
	int type, n;
        double tlmin, tlmax, binsiz;
	double tscale, tzero;
	int scaled, exists, settype;
#endif
{
  CalcCols sym, cur;  
  int got;
  int enter=1;

  if( (sym=_CalcColLookup(cname)) ){
    if( !settype ) return sym;
    enter=0;
  }
  else{
    /* allocate a new symbol record */
    if( !(sym = (CalcCols)xcalloc(1, sizeof(CalcColRec))) )
      return NULL;
  }

  /* fill in the blanks */
  if( sym->name ) xfree(sym->name);
  sym->name = xstrdup(name);
  if( sym->sname ) xfree(sym->sname);
  sym->sname = xstrdup(sname);
  if( sym->cname ) xfree(sym->cname);
  sym->cname = xstrdup(cname);
  sym->type = type;
  sym->n = n;
  sym->tlmin = tlmin;
  sym->tlmax = tlmax;
  sym->binsiz = binsiz;
  sym->tscale = tscale;
  sym->tzero = tzero;
  sym->scaled = scaled;
  sym->exists = exists;
  sym->settype = settype;

  /* get tdim value, if there is one */
  if( sym->exists && (sym->n>1) )
    sym->tdim = FunParamGets(fun, "TDIM", exists, NULL, &got);

  /* add to list, maintaining order */
  if( enter ){
    if( !columns ){
      columns = sym;
    }
    else{
      for(cur=columns; cur->next!=NULL; cur=cur->next)
        ;
      cur->next = sym;
    }
  }
  return sym;
}

/* process a column */
#ifdef YY_USE_PROTOS
static int
_CalcColProc(char *s, int expl)
#else
static int _CalcColProc(s, expl)
     char *s;
     int expl;
#endif
{
  char *e;
  char *t;
  char name[SZ_LINE];
  char sname[SZ_LINE];
  char cname[SZ_LINE];
  char aname[SZ_LINE];
  char fname[SZ_LINE];
  int ptype;
  int poff;
  int exists;
  int dims;
  int settype=0;
  int type=0;
  int n=0;
  int scaled=0;
  double tlmin=0.0, tlmax=0.0, binsiz=1.0;  
  double tscale=1.0, tzero=0.0;

  /* save expression */
  e = s;

  /* gather struct name */
  for(t=sname; *s && (*s != '-');)
    *t++ = *s++;
  *t = '\0';
  /* skip past "->" */
  if( *s && (*s == '-') ) s += 2;

  /* if this is not a special name, just append it and exit */
  if( !_CalcColName(sname) ){
    if( !expl ){
      _CalcCat(e, &expr, &nexpr);
      return 1;
    }
  }

  /* gather column name */
  for(t=cname; *s && (*s != '[') && (*s != ':');)
    *t++ = *s++;
  *t = '\0';

  /* explicit columns don't have the prefix */
  if( expl && *sname && !*cname ){
    strncpy(cname, sname, SZ_LINE);
    strncpy(sname, "cur", SZ_LINE);
  }

  /* gather array part */
  if( *s && (*s == '[') ){
    for(t=aname; *s && (*s != ':');)
      *t++ = *s++;
    *t = '\0';
  }
  else
    *aname = '\0';
  if( *aname )
    snprintf(fname, SZ_LINE, "%s->%s%s", sname, cname, aname);
  else
    snprintf(fname, SZ_LINE, "%s->%s", sname, cname);

  /* the expression name is the combination of sname (which we will set to
     a struct we know about) and cname (which is a struct member name) */
  snprintf(name, SZ_LINE, "%s->%s", sname, cname);

  /* see if this column exists or if its new */
  exists = FunColumnLookup(ifun, cname, 0, NULL, &type, NULL, NULL, &n, NULL);

  /* gather up specifier info */
  for(; *s; s++){
    if( *s == ':' ){
      _FunColumnType(s+1, &type, &n, &tlmin, &tlmax, &binsiz, &dims,
			  &tscale, &tzero, &scaled, &ptype, &poff);
      settype = 1;
      break;
    }
  }
  if( !exists && !settype && !_CalcColLookup(cname) )
    _calcerror("new column requires a type");
  if( exists && settype )
    doreplace=1;

  /* add this column to the list */
  if(_CalcColNew(ifun, name, sname, cname,
                 type, n, tlmin, tlmax, binsiz, 
		 tscale, tzero, scaled,
		 exists, settype)){
    if( !expl ) _CalcCat(fname, &expr, &nexpr);
    return 1;
  }
  else{
    return _calcerror("could not enter new funcalc column");
  }  
}

/*
 *
 * Semi-public Routines 
 *
 *
 */

/*
 *
 * _CalcCat -- add a string to the filter string buffer
 *
 */
#ifdef YY_USE_PROTOS
void
_CalcCat(char *str, char **ostr, int *olen)
#else
void _CalcCat(str, ostr, olen)
     char *str;
     char **ostr;
     int *olen;
#endif
{
  int blen;
  int slen;
  
  if( (str == NULL) || (*str == '\0') )
    return;
  else
    slen = strlen(str) + 1;
  
  if( (*ostr == NULL) || (**ostr == '\0') )
    blen = 0;
  else
    blen = strlen(*ostr);
  
  while( (blen + slen) >= *olen ){
    *olen += SEGINC;
  }
  if( blen == 0 )
    *ostr = (char *)xcalloc(*olen, sizeof(char));
  else
    *ostr = (char *)xrealloc(*ostr, *olen);
  strcat(*ostr, str);
}

#ifdef YY_USE_PROTOS
char *
FunCalcParse(char *iname, char *oname,
             char *cmd, char *ex, char *autod, int narg)
#else
char *FunCalcParse(iname, oname, cmd, ex, autod, narg)
    char *iname;
    char *oname;
    char *cmd;
    char *ex;
    char *autod;
    int narg;
#endif
{
  int i;

  /* avoid -W unused parameter warning */
  if( 0 ) oname = oname;

  /* initialize global variables */
  n = 0;
  rectype = REC_CUR;
  var = 1;
  args = narg;

  /* save command */
  if( command ) xfree(command);
  command = xstrdup(cmd);

  /* save autodeclare */
  if( autodeclare ) xfree(autodeclare);
  autodeclare = xstrdup(autod);

  /* open the input FITS file */
  if( !(ifun = FunOpen(iname, "rc", NULL)) )
    gerror(stderr, "can't FunOpen input file (or find extension): %s\n",
	   iname);

  /* parse expression */
  calc_scan_string(ex);
  calclex();

  /* done with input FITS file */
  FunClose(ifun);
  /* free up space */
  if( expr ){
    xfree(expr);
    expr = NULL;
  }
  nexpr = 0;
  for(i=0; i<MAXCODE; i++){
    if( code[i] ){
      xfree(code[i]);
      code[i] = NULL;
    }
    ncode[i] = 0;
  }
  if( mbuf ){
    xfree(mbuf);
    mbuf = NULL;
    mlen = 0;
  }
  /* return resulting program */
  return prog;
}

#ifdef YY_USE_PROTOS
int
_calcerror(char *msg)
#else
int _calcerror(msg)
     char *msg;
#endif
{
  if( *calctext )
    gerror(stderr,  "%s while processing '%s'\n",
           msg ? msg : "filterr", calctext);
  else
    gerror(stderr,  "%s\n", msg ? msg : "filterr");
  YY_FLUSH_BUFFER;
  yyterminate();
}

#ifdef YY_USE_PROTOS
int calcwrap(void)
#else
int calcwrap()
#endif
{
  return 1;
}

